/*
 * file: libpv.h
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#ifndef LIBPV_H
#define LIBPV_H

#include <limits.h>
#include <stdio.h>
#include <inttypes.h>

#define PV_PATH_MAX	NAME_MAX

# ifdef __cplusplus
extern "C"
{
# endif

#define	LIBPV_STORE_VARNAME	"LIBPV_STORE"
#define LIBPV_DESC_VARNAME	"LIBPV_DESC"
#define LIBPV_STORETYPE_VARNAME	"LIBPV_STORETYPE"

typedef struct {
	char	_data[68];
} pv_engine_t;


typedef enum pv_bs_store_type {
	PV_STORE_STANDARD,
	PV_STORE_8BIT,
} pv_store_type_t;


typedef struct pv_engine_attr {
	char			desc_filename[NAME_MAX + 1];
	char			store_filename[NAME_MAX + 1];
	pv_store_type_t		store_type;
	int			is_server;
	uint16_t		jvisu_port;
} pv_engine_attr_t;

struct pv;

typedef int	(*pv_update_callback_fn_t)
					(pv_engine_t		*engine,
					 struct pv		*pv,
					 unsigned long		data);

/* start/stop/stuff */
extern int	pv_init			(pv_engine_t		*engine,
					 pv_engine_attr_t	*attr);

extern int	pv_destroy		(pv_engine_t		*engine_anon);

extern int	pv_engine_attr_init	(pv_engine_attr_t	*attr);

extern int	pv_engine_attr_init_from_cmdline
					(pv_engine_attr_t	*attr,
					 int			argc,
					 char			*argv[]);

extern int	pv_engine_attr_init_from_env
					(pv_engine_attr_t	*attr);

extern int	pv_debuglevel		(pv_engine_t		*engine,
					 int			level);


/* registration stuff */
extern uint8_t	*pv_register_uint8	(pv_engine_t		*engine,
					 const char		*varname);

extern int8_t	*pv_register_int8	(pv_engine_t		*engine,
					 const char		*varname);

extern uint16_t	*pv_register_uint16	(pv_engine_t		*engine,
					 const char		*varname);

extern int16_t	*pv_register_int16	(pv_engine_t		*engine,
					 const char		*varname);

extern uint32_t	*pv_register_uint32	(pv_engine_t		*engine,
					 const char		*varname);

extern int32_t	*pv_register_int32	(pv_engine_t		*engine,
					 const char		*varname);

extern float	*pv_register_float	(pv_engine_t		*engine,
					 const char		*varname);

extern double	*pv_register_double	(pv_engine_t		*engine,
					 const char		*varname);

extern char	*pv_register_string	(pv_engine_t		*engine,
					 const char		*varname);

extern uint8_t	*pv_register_uint8_array (pv_engine_t		*engine,
					 const char		*varname);

extern int8_t	*pv_register_int8_array	(pv_engine_t		*engine,
					 const char		*varname);

extern uint16_t	*pv_register_uint16_array
					(pv_engine_t		*engine,
					 const char		*varname);

extern int16_t	*pv_register_int16_array(pv_engine_t		*engine,
					 const char		*varname);

extern uint32_t	*pv_register_uint32_array
					(pv_engine_t		*engine,
					 const char		*varname);

extern int32_t	*pv_register_int32_array
					(pv_engine_t		*engine,
					 const char		*varname);

extern float	*pv_register_float_array
					(pv_engine_t		*engine,
					 const char		*varname);

extern double	*pv_register_double_array
					(pv_engine_t		*engine,
					 const char		*varname);


/* string handling stuff */
extern int	pv_cmp_string		(pv_engine_t		*engine,
					 const char		*string,
					 char			*s2);

extern int	pv_get_string		(pv_engine_t		*engine,
					 const char		*string,
					 char			*dest,
					 size_t			n);

extern int	pv_set_string		(pv_engine_t		*engine,
					 char			*string,
					 char			*src);

/* extern int	pv_set_nstring		(pv_engine_t		*engine, */
/* 					 char			*string, */
/* 					 char			*src, */
/* 					 size_t			n); */

/* set-a-default */
extern int	pv_default		(pv_engine_t		*engine,
					 void			*var);

/* transfer pv from and to device */
extern int	pv_to_device		(pv_engine_t		*engine,
					 void			*var);

extern int	pv_from_device		(pv_engine_t		*engine,
					 void			*var);

extern int	pv_set_fn_read		(pv_engine_t		*engine,
					 void			*var,
					 pv_update_callback_fn_t
								device_read);

extern int	pv_set_fn_write		(pv_engine_t		*engine,
					 void			*var,
					 pv_update_callback_fn_t
								device_write);

extern int	pv_device_read_int_file_ascii
					(pv_engine_t		*engine,
					 struct pv		*pv,
					 unsigned long 		data);

extern int	pv_device_write_int_file_ascii
					(pv_engine_t		*engine,
					 struct pv		*pv,
				  	 unsigned long 		data);

/* lock-stuff */
extern void	pv_lock			(pv_engine_t		*engine);
extern void	pv_unlock		(pv_engine_t		*engine);
extern void	pv_flush		(pv_engine_t		*engine,
					 void			*var,
					 size_t			n);

/* event stuff */
extern int	pv_event_register	(const pv_engine_t	*engine,
					 const void		*pv_var);

extern int	pv_event_deregister	(const pv_engine_t	*engine,
					 const void		*pv_var);

extern int	pv_event_await_syncronous
					(const pv_engine_t	*engine,
					 void			**pv_var);

extern int	pv_event_generate	(const pv_engine_t	*engine_anon,
					 const void		*pv_var);
# ifdef __cplusplus
}
# endif

#endif	/* !LIBPV_H */
