/*
 * file: pv_default.h
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#ifndef PV_DEFAULT_H
#define PV_DEFAULT_H

#include <pv_types.h>

static inline void pv_default_pv(struct pv_engine	*engine,
				 struct pv		*pv)
{
	union pv_data	*data = engine->store.map + pv->offset;

	switch (pv->type) {
	case PV_TYPE_INT8:
		*data->s8 = pv->dflt.s8;
		break;
	case PV_TYPE_UINT8:
		*data->u8 = pv->dflt.u8;
		break;

	case PV_TYPE_INT16:
		*data->s16 = pv->dflt.s16;
		break;
	case PV_TYPE_UINT16:
		*data->u16 = pv->dflt.u16;
		break;

	case PV_TYPE_INT32:
		*data->s32 = pv->dflt.s32;
		break;
	case PV_TYPE_UINT32:
		*data->u32 = pv->dflt.u32;
		break;

	case PV_TYPE_INT64:
		*data->s64 = pv->dflt.s64;
		break;
	case PV_TYPE_UINT64:
		*data->u64 = pv->dflt.u64;
		break;

	case PV_TYPE_STRING:
		memcpy(data->string, pv->dflt.string, pv->size);
		break;

	case PV_TYPE_FLOAT:
		*data->f = pv->dflt.f;
		break;
	
	case PV_TYPE_DOUBLE:
		*data->d = pv->dflt.d;
		break;

	case PV_TYPE_INT8_ARRAY:
	case PV_TYPE_UINT8_ARRAY:
	case PV_TYPE_INT16_ARRAY:
	case PV_TYPE_UINT16_ARRAY:
	case PV_TYPE_INT32_ARRAY:
	case PV_TYPE_UINT32_ARRAY:
	case PV_TYPE_FLOAT_ARRAY:
	case PV_TYPE_DOUBLE_ARRAY:
		if (pv->dflt.ptr != NULL)
			memcpy(data->ptr, pv->dflt.ptr, pv->size * pv->arraylen);
		break;

	case PV_TYPE_UNDEF:
		break;
	}
}

#endif	/* !PV_DEFAULT_H */
